/*--------------------------------------------------------------------------------*
 *                                                                                *
 * CodecPro Incorporated                                                          *
 * 2162 Av Laurier E                                                              *
 * Montreal (Quebec)                                                              *
 * Canada, H2H 1C2                                                                *
 *                                                                                *
 * www.codecpro.com                                                               *
 *                                                                                *
 * This material is subject to continuous developments and improvements. All      *
 * warranties implied or expressed, including but not limited to, implied         *
 * warranties of merchantability, fitness for purpose, condition of title,        *
 * noninfringement, are excluded. In no event shall CodecPro Incorporated and its *
 * suppliers be liable for any special, indirect or consequential damages or any  *
 * damages whatsoever arising out of or in connection with the use of this        *
 * information. The foregoing disclaimer shall apply to the maximum extent        *
 * permitted by applicable law, even if any remedy fails its essential purpose.   *
 *                                                                                *
 *                     NON-COMMERCIAL use only.                                   *
 *                                                                                *
 * For a commercial version please contact CodecPro at sales@codecpro.com         *
 *                                                                                *
 *--------------------------------------------------------------------------------*/

/*
 *===================================================================
 *  3GPP AMR Wideband Floating-point Speech Codec
 *===================================================================
 */

#define _CRT_SECURE_NO_WARNINGS

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "typedef.h"
#include "cp_amrwb.h"

/*
 * Define AMRWB_MAGIC_NUMBER to add "#!AMR-WB\n" at the beginning of the encoded output file
 */
//#define AMRWB_MAGIC_NUMBER "#!AMR-WB\n"


/*
 * ENCODER.C
 *
 *    Usage : encoder (-dtx) mode speech_file  bitstream_file
 *
 *    Format for speech_file:
 *      Speech is read from a binary file of 16 bits data.
 *
 *    Format for bitstream_file:
 *        Described in RFC4867 section 5.1 and 5.3
 *
 *    mode = 0..8 (bit rate = 6.60 to 23.85 k)
 *
 *    -dtx if DTX is ON
 */

int main(int argc, char *argv[])
{
   FILE *f_speech = NULL;                 /* File of speech data                   */
   FILE *f_serial = NULL;                 /* File of serial bits for transmission  */

   Word32 serial_size, frame = 0;
   Word16 signal[L_FRAME16k];             /* Buffer for speech @ 16kHz             */
   Word16 allow_dtx = 0, mode = 0;
   UWord8 serial[NB_SERIAL_MAX];

   printf("\n");
   printf("**************       CodecPro Incorporated        ************\n");
   printf("\n");
   printf("---------------- AMR-WB Floating point encoder ---------------\n");
   printf("\n");
   printf("----------------   (Non-Commercial use only)   ---------------\n");
   printf("\n");


   /*
    * Open speech file and result file (output serial bit stream)
    */

   if ((argc > 1) && (strcmp(argv[1], "-dtx") == 0))
   {
      allow_dtx = 1;
      argv[1] = argv[0];   // move exe name in case Usage display is needed
      argv++;
      argc--;
   }

   if (argc != 4)
   {
      fprintf(stderr, "Usage : %s (-dtx) mode speech_file  bitstream_file\n", argv[0]);
      fprintf(stderr, "\n");
      fprintf(stderr, "Format for speech_file:\n");
      fprintf(stderr, "  Speech is read form a binary file of 16 bits data.\n");
      fprintf(stderr, "\n");
      fprintf(stderr, "Format for bitstream_file:\n");
      fprintf(stderr, "  Described in RFC 4867 (Sections 5.1 and 5.3).\n");
      fprintf(stderr, "\n");
      fprintf(stderr, " ===================================================================\n");
      fprintf(stderr, " mode   :  (0)  (1)   (2)   (3)   (4)   (5)   (6)   (7)   (8)     \n");
      fprintf(stderr, " bitrate: 6.60 8.85 12.65 14.25 15.85 18.25 19.85 23.05 23.85 kbit/s\n");
      fprintf(stderr, " ===================================================================\n");
      fprintf(stderr, "\n");
      fprintf(stderr, "-dtx if DTX is ON, default is OFF\n");
      fprintf(stderr, "\n");
      return(0);
   }

   mode = (Word16) atoi(argv[1]);

   /*
    * Validate input arguments
    */
   if ((mode < 0) || (mode > 8))
   {
      fprintf(stderr, " error in bit rate mode %d: use 0 to 8\n", mode);
      return(0);
   }
   if ((f_speech = fopen(argv[2], "rb")) == NULL)
   {
      fprintf(stderr, "Error opening input file  %s !!\n", argv[2]);
      return(0);
   }
   if ((f_serial = fopen(argv[3], "wb")) == NULL)
   {
      fprintf(stderr, "Error opening output bitstream file %s !!\n", argv[3]);
      return(0);
   }

   fprintf(stderr, "    Input speech file:  %s\n", argv[2]);
   fprintf(stderr, "Output bitstream file:  %s\n", argv[3]);
   fprintf(stderr, "                  dtx:  %d\n", allow_dtx);
   fprintf(stderr, "                 mode:  %d\n", mode);

   /*
    * Initialisation
    */

   if (E_IF_amrwb_init() != 0)
   {
      fprintf(stderr, "Encoder initialization error!!\n");
      return(0);
   }


#ifdef AMRWB_MAGIC_NUMBER
   /*
    * Write the magic number at the beginning of the bitstream file
	 */
	fwrite(AMRWB_MAGIC_NUMBER, sizeof(char), strlen(AMRWB_MAGIC_NUMBER), f_serial);
#endif

   /*
    * Loop for every analysis/transmission frame.
    *   -New L_FRAME data are read. (L_FRAME = number of speech data per frame)
    *   -Call coder to encode the speech.
    *   -The compressed serial output stream is written to a file.
    */

   while (fread(signal, sizeof(Word16), L_FRAME16k, f_speech) == L_FRAME16k)
   {

      serial_size = E_IF_amrwb_encode(mode, signal, serial, allow_dtx);

      fwrite(serial, 1, serial_size, f_serial);

      fprintf(stderr, "     Frames processed:  %ld\r", frame++);
   }

   fprintf(stderr, "\n");
   E_IF_amrwb_exit();   // free allocated memory

   fclose(f_speech);
   fclose(f_serial);

   return 0;
}
